package com.lzmispt.data.resource.service.resources.impl;

package .tmp;

import com.alibaba.csp.sentinel.util.StringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Joiner;
import com.lzmispt.core.entity.dto.authorize.UserInfoDTO;
import com.lzmispt.core.entity.dto.querymodel.QueryModelDTO;
import com.lzmispt.core.entity.dto.querymodel.SortModelDTO;
import com.lzmispt.core.service.query.IQuery;
import com.lzmispt.data.center.entity.dto.databasestruct.ColumnParamDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.DBColumnDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.DataSourceDTO;
import com.lzmispt.data.center.entity.dto.databasestruct.TableParamDTO;
import com.lzmispt.data.center.entity.dto.log.CommonLogDTO;
import com.lzmispt.data.center.entity.dto.querymodel.DCQueryModelDTO;
import com.lzmispt.data.center.entity.dto.resource.file.DeleteFileTableDTO;
import com.lzmispt.data.center.entity.dto.resource.metadata.DeleteFileTableMsg;
import com.lzmispt.data.center.entity.dto.resource.metadata.FileTableUsedInfoDTO;
import com.lzmispt.data.center.entity.enums.BaseSysFieldEnum;
import com.lzmispt.data.center.entity.enums.FileSysFieldEnum;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.center.entity.vo.dcinterface.InterfaceEntityRelationVO;
import com.lzmispt.data.center.entity.vo.resource.datasource.DataSourceVO;
import com.lzmispt.data.center.entity.vo.resource.file.FileInfoTableVO;
import com.lzmispt.data.center.entity.vo.resource.file.FileTableUsedInterfaceVO;
import com.lzmispt.data.center.entity.vo.resource.metadata.FileTableUsageVO;
import com.lzmispt.data.center.entity.vo.resource.metadata.MetadataFieldVO;
import com.lzmispt.data.center.entity.vo.resource.resources.ResFileInfoTableVO;
import com.lzmispt.data.center.entity.vo.standard.metadata.MetadataTypeVO;
import com.lzmispt.data.resource.client.DataInterfaceClient;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.dao.developconfig.impl.DevelopConfigDaoImpl;
import com.lzmispt.data.resource.dao.metadata.IMetadataDao;
import com.lzmispt.data.resource.dao.resources.IResourceFileInfoFieldDao;
import com.lzmispt.data.resource.dao.resources.IResourceFileInfoTableDao;
import com.lzmispt.data.resource.entity.dto.file.AddFileInfoTableDTO;
import com.lzmispt.data.resource.entity.dto.file.CheckFileInfoDTO;
import com.lzmispt.data.resource.entity.dto.file.QueryFileInfoTableDTO;
import com.lzmispt.data.resource.entity.dto.file.UpdateFileInfoTableDTO;
import com.lzmispt.data.resource.entity.dto.metadata.TableStructInfoDTO;
import com.lzmispt.data.resource.entity.po.developconfig.DevelopConfigPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataDirectoryPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityPO;
import com.lzmispt.data.resource.entity.po.resources.ResFileInfoFieldPO;
import com.lzmispt.data.resource.entity.po.resources.ResFileInfoTablePO;
import com.lzmispt.data.resource.service.history.HistoryTableManager;
import com.lzmispt.data.resource.service.metadata.IInfoItemSync;
import com.lzmispt.data.resource.service.resources.IResFileInfoTable;
import com.lzmispt.dcbase.ServiceUtil;
import com.lzmispt.dcbase.database.IManage;
import com.lzmispt.dcbase.databasestruct.DataStructProvider;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.dcbase.sysfield.SysFieldHelper;
import com.lzmispt.dcbase.util.HeaderInterceptor;
import com.lzmispt.dcbase.util.TokenUtil;
import com.lzmispt.framework.entity.entity.ResultEntity;
import com.lzmispt.framework.exception.ExceptionUtil;
import com.lzmispt.framework.fconst.FrameConst;
import com.lzmispt.framework.log.LogHelper;
import com.lzmispt.framework.utils.DBKeywordUtil;
import com.lzmispt.framework.utils.RabbitMQUtil;
import com.lzmispt.framework.utils.WorkerId;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

@RefreshScope
public abstract class ResFileInfoTableManager implements IResFileInfoTable, IManage {
  private static final Integer CN_NAME_MAX_LENGTH = Integer.valueOf(100);
  
  private static final Integer CODE_MAX_LENGTH = Integer.valueOf(100);
  
  private static final Integer DESCRIPTION_MAX_LENGTH = Integer.valueOf(500);
  
  private static final String CN_NAME_OVERLONG_MSG = "文件表中文名长度超过" + CN_NAME_MAX_LENGTH;
  
  private static final String CODE_OVERLONG_MSG = "文件表编码长度超过" + CODE_MAX_LENGTH;
  
  private static final String DESCRIPTION_OVERLONG_MSG = "文件表备注长度超过" + DESCRIPTION_MAX_LENGTH;
  
  private static final Integer FIELD_LENGTH_10 = Integer.valueOf(10);
  
  private static final Integer FIELD_LENGTH_20 = Integer.valueOf(20);
  
  private static final Integer FIELD_LENGTH_36 = Integer.valueOf(36);
  
  private static final Integer FIELD_LENGTH_50 = Integer.valueOf(50);
  
  private static final Integer FIELD_LENGTH_500 = Integer.valueOf(500);
  
  private static final Integer FIELD_LENGTH_2000 = Integer.valueOf(2000);
  
  private static final Integer FIELD_LENGTH_4000 = Integer.valueOf(4000);
  
  private static final Integer COLUMN_LENGTH_10 = Integer.valueOf(10);
  
  private static final Integer COLUMN_LENGTH_20 = Integer.valueOf(20);
  
  private static final Integer COLUMN_LENGTH_36 = Integer.valueOf(36);
  
  private static final Integer COLUMN_LENGTH_50 = Integer.valueOf(50);
  
  private static final Integer COLUMN_LENGTH_500 = Integer.valueOf(500);
  
  private static final Integer COLUMN_LENGTH_2000 = Integer.valueOf(2000);
  
  private static final Integer COLUMN_LENGTH_4000 = Integer.valueOf(4000);
  
  final int oracleMaxLength = 2000;
  
  @Autowired
  DataStandardClient dataStandardClient;
  
  @Autowired
  DataInterfaceClient dataInterfaceClient;
  
  @Autowired
  private HistoryTableManager historyTableManager;
  
  @Autowired
  private DevelopConfigDaoImpl developConfigDao;
  
  @Value("${data-center.data-authority.collection}")
  private String collectionAuthority;
  
  @Value("${data-center.data-authority.manage}")
  private String manageAuthority;
  
  @Value("${data-center.data-mode:mysql}")
  private String dbType;
  
  @Value("${data-center.primary-key-mode}")
  private String primaryKeyMode;
  
  @Autowired
  private IQuery query;
  
  public ResultEntity<Map> selectFileInfoTableList(QueryFileInfoTableDTO queryDataInterfaceDTO) {
    String authority = getDefaultCode().equals("Collection") ? this.collectionAuthority : this.manageAuthority;
    IResourceFileInfoTableDao resFileInfoTableDao = (IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class);
    Map<String, Object> param = new HashMap<>();
    Map<String, Object> resultMap = new HashMap<>();
    if ("org".equals(authority)) {
      param.put("organizationId", queryDataInterfaceDTO.getOrganizationId());
    } else if ("userorg".equals(authority)) {
      param.put("userId", queryDataInterfaceDTO.getUserId());
      param.put("organizationId", queryDataInterfaceDTO.getOrganizationId());
    } 
    DCQueryModelDTO queryModel = queryDataInterfaceDTO.getQuery();
    String whereSql = null;
    if (queryModel != null) {
      List<SortModelDTO> sortModels = queryModel.getSortModels();
      if (sortModels != null)
        for (SortModelDTO sortModel : sortModels) {
          if ("name".equals(sortModel.getFieldName())) {
            param.put("needJoin", Boolean.valueOf(true));
            break;
          } 
        }  
      boolean isEscapeStr = false;
      if (this.dbType.equalsIgnoreCase("sqlserver"))
        isEscapeStr = true; 
      whereSql = (String)this.query.getWhereSqlStr((QueryModelDTO)queryModel, this.dbType, true, isEscapeStr).getObj();
      if (!StringUtil.isEmpty(whereSql) && queryModel.getFilterModel() != null)
        whereSql = " and " + whereSql; 
      param.put("query", whereSql);
    } 
    List<ResFileInfoTablePO> selectList = resFileInfoTableDao.selectFileInfoTableList(param);
    List<ResFileInfoTableVO> list = new ArrayList<>();
    ResFileInfoTableVO fileInfoTableVO = null;
    if (!CollectionUtils.isEmpty(selectList))
      for (ResFileInfoTablePO item : selectList) {
        fileInfoTableVO = new ResFileInfoTableVO();
        BeanUtils.copyProperties(item, fileInfoTableVO);
        list.add(fileInfoTableVO);
      }  
    resultMap.put("list", list);
    if (queryDataInterfaceDTO.getQuery().getPageSize() != null) {
      queryDataInterfaceDTO.getQuery().setPageSize(null);
      queryDataInterfaceDTO.getQuery().setPageIndex(null);
      queryDataInterfaceDTO.getQuery().setSortModels(null);
      boolean isEscapeStr = false;
      if (this.dbType.equalsIgnoreCase("sqlserver"))
        isEscapeStr = true; 
      whereSql = (String)this.query.getWhereSqlStr((QueryModelDTO)queryModel, this.dbType, true, isEscapeStr).getObj();
      if (!StringUtil.isEmpty(whereSql) && queryModel.getFilterModel() != null)
        whereSql = " and " + whereSql; 
      param.put("query", whereSql);
      int total = resFileInfoTableDao.selectFileInfoTableListCount(param);
      resultMap.put("total", Integer.valueOf(total));
    } 
    return ResultEntity.successMessage(resultMap);
  }
  
  public ResultEntity<Integer> getNewTableOrder(Integer dataCategory) {
    IResourceFileInfoTableDao resFileInfoTableDao = (IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class);
    Integer maxOrder = resFileInfoTableDao.getFileTableOrderMax(dataCategory);
    return ResultEntity.successMessage(Integer.valueOf((maxOrder == null) ? 1 : ((maxOrder.intValue() == Integer.MAX_VALUE) ? Integer.MAX_VALUE : (maxOrder.intValue() + 1))));
  }
  
  public ResultEntity<Long> insertFileInfoTable(AddFileInfoTableDTO addFileInfoTableDTO, String tokenId) {
    if (!StringUtils.isEmpty(addFileInfoTableDTO.getFileTableCode()) && addFileInfoTableDTO.getFileTableCode().length() > CODE_MAX_LENGTH.intValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_EXCEEDS_HUNDRED.getCode()), CODE_OVERLONG_MSG); 
    if (!StringUtils.isEmpty(addFileInfoTableDTO.getDescription()) && addFileInfoTableDTO.getDescription().length() > DESCRIPTION_MAX_LENGTH.intValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_REMARK_EXCEEDS_FIVE_HUNDRED.getCode()), DESCRIPTION_OVERLONG_MSG); 
    if (StringUtils.isEmpty(addFileInfoTableDTO.getFileTableCode()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_IS_NULL.getCode()), StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_IS_NULL.getMessage()); 
    IResourceFileInfoTableDao resFileInfoTableDao = (IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class);
    List<ResFileInfoTablePO> fileTableList = resFileInfoTableDao.getFileTableByCode(addFileInfoTableDTO.getFileTableCode(), null);
    if (fileTableList.size() > 0)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_EXIST.getMessage()); 
    if (addFileInfoTableDTO.getFileTableOrder() == null) {
      Integer maxOrder = resFileInfoTableDao.getFileTableOrderMax(addFileInfoTableDTO.getDataCategory());
      addFileInfoTableDTO.setFileTableOrder(Integer.valueOf((maxOrder == null) ? 1 : ((maxOrder.intValue() == Integer.MAX_VALUE) ? Integer.MAX_VALUE : (maxOrder.intValue() + 1))));
    } 
    ResFileInfoTablePO fileInfoTablePO = new ResFileInfoTablePO();
    Long fileTableId = (Long)WorkerId.getNextId(Long.class);
    BeanUtils.copyProperties(addFileInfoTableDTO, fileInfoTablePO);
    fileInfoTablePO.setFileTableId(fileTableId);
    DevelopConfigPO developConfig = this.developConfigDao.getDevelopConfig(getDefaultCode());
    if (developConfig != null) {
      JSONObject jsonObject = JSONObject.parseObject(developConfig.getConfigPlatJson());
      String fileRootPath = (jsonObject != null && !StringUtils.isEmpty(jsonObject.getString("fileRootPath"))) ? jsonObject.getString("fileRootPath") : "DataCenter";
      fileInfoTablePO.setFileRootPath(fileRootPath);
    } else {
      fileInfoTablePO.setFileRootPath("DataCenter");
    } 
    fileInfoTablePO.setUserIdRecord(addFileInfoTableDTO.getUserId());
    fileInfoTablePO.setOrgIdRecord(addFileInfoTableDTO.getOrganizationId());
    fileInfoTablePO.setCreateTime(new Date());
    fileInfoTablePO.setUpdateTime(new Date());
    fileInfoTablePO.setIsDelete(DCConst.FALSE);
    if (!addFileInfoField(fileInfoTablePO.getFileTableId(), addFileInfoTableDTO.getUserId(), addFileInfoTableDTO.getOrganizationId()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CREATE_FILE_INFO_TABLE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_CREATE_FILE_INFO_TABLE_FAILED.getMessage()); 
    if (resFileInfoTableDao.insertFileInfoTable(fileInfoTablePO) > 0) {
      logSystemInfo("新增文件信息表 文件表编码:【" + addFileInfoTableDTO.getFileTableCode() + "】,文件表中文名:【" + addFileInfoTableDTO.getFileTableName() + "】", getDefaultCode(), tokenId);
      return ResultEntity.successMessage(fileInfoTablePO.getFileTableId());
    } 
    return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_ADD_FILE_TABLE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_ADD_FILE_TABLE_FAILED.getMessage());
  }
  
  public boolean checkHaveFileTableNameInDB(DataStructProvider dataStructProvider, String fileTableName) {
    return dataStructProvider.isExistsTable(fileTableName).booleanValue();
  }
  
  public ResultEntity<Boolean> checkFileTableNameIsExist(CheckFileInfoDTO checkFileInfoDTO, String tokenId) {
    IResourceFileInfoTableDao resFileInfoTableDao = (IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class);
    DataSourceVO dataSourceVO = null;
    if (checkFileInfoDTO.getDataSourceId() != null)
      dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(checkFileInfoDTO.getDataSourceId(), getDefaultCode(), tokenId).getObj(); 
    if (dataSourceVO == null) {
      logErrorInfo(null, "校验文件表英文名出错, 根据Id获取配置的数据库实例失败! 数据源id:" + checkFileInfoDTO.getDataSourceId(), getDefaultCode(), tokenId);
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_FAILED
          .getMessage());
    } 
    if (DBKeywordUtil.isDBKeyword(checkFileInfoDTO.getTableName(), dataSourceVO.getType()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_NAME_IS_KEYWORD.getCode()), 
          String.format(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_NAME_IS_KEYWORD.getMessage(), new Object[] { checkFileInfoDTO.getTableName() }), Boolean.valueOf(true)); 
    List<ResFileInfoTablePO> list = resFileInfoTableDao.getTableByNameAndDataSourceId(checkFileInfoDTO.getDataSourceId(), checkFileInfoDTO.getTableName(), checkFileInfoDTO.getFileTableId());
    if (list != null && list.size() > 0)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_NAME_EXISTS.getCode()), 
          String.format(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_NAME_EXISTS.getMessage(), new Object[] { checkFileInfoDTO.getTableName() }), Boolean.valueOf(true)); 
    DataSourceDTO dataSourceDTO = new DataSourceDTO();
    BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
    ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
    if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
      return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
    DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
    if (checkHaveFileTableNameInDB(dataStructProvider, checkFileInfoDTO.getTableName()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_NAME_EXISTS.getCode()), 
          String.format(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_NAME_EXISTS.getMessage(), new Object[] { checkFileInfoDTO.getTableName() }), Boolean.valueOf(true)); 
    return ResultEntity.successMessage(Boolean.valueOf(false));
  }
  
  public boolean addFileInfoField(Long fileTableId, Long userId, Long organizationId) {
    Object object = new Object(this, fileTableId, userId, organizationId);
    if (DCConst.PrimaryKeyMode.GUID.equals(this.primaryKeyMode)) {
      object.add(new ResFileInfoFieldPO((Long)WorkerId.getNextId(Long.class), fileTableId, SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.DBID), "数据表主键id", "String", FIELD_LENGTH_36, Integer.valueOf(0), Integer.valueOf(1), Integer.valueOf(0), Integer.valueOf(object.size() + 1), userId, organizationId));
    } else {
      object.add(new ResFileInfoFieldPO((Long)WorkerId.getNextId(Long.class), fileTableId, SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.DBID), "数据表主键id", "Long", FIELD_LENGTH_20, Integer.valueOf(0), Integer.valueOf(1), Integer.valueOf(0), Integer.valueOf(object.size() + 1), userId, organizationId));
    } 
    int count = ((IResourceFileInfoFieldDao)ServiceUtil.getResFileInfoFieldDao(getDefaultCode(), IResourceFileInfoFieldDao.class)).insertFileInfoFieldList((List)object);
    return (count > 0);
  }
  
  public ResultEntity<Boolean> updateFileInfoTable(UpdateFileInfoTableDTO updateFileInfoTableDTO, String tokenId) {
    if (!StringUtils.isEmpty(updateFileInfoTableDTO.getFileTableCnName()) && updateFileInfoTableDTO.getFileTableCnName().length() > CN_NAME_MAX_LENGTH.intValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CN_NAME_EXCEEDS_HUNDRED.getCode()), CN_NAME_OVERLONG_MSG); 
    if (!StringUtils.isEmpty(updateFileInfoTableDTO.getDescription()) && updateFileInfoTableDTO.getDescription().length() > DESCRIPTION_MAX_LENGTH.intValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_REMARK_EXCEEDS_FIVE_HUNDRED.getCode()), DESCRIPTION_OVERLONG_MSG); 
    if (StringUtils.isEmpty(updateFileInfoTableDTO.getFileTableCode()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_IS_NULL.getCode()), StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_IS_NULL.getMessage()); 
    ResFileInfoTablePO updateFileInfoTablePO = new ResFileInfoTablePO();
    ResFileInfoTablePO resFileInfoTablePO = ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).selectFileInfoTable(updateFileInfoTableDTO.getFileTableId());
    if (checkFileTableCodeExist(resFileInfoTablePO, updateFileInfoTableDTO))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_EXIST.getCode()), StatusCodeEnum.DATA_RESOURCE_FILE_TABLE_CODE_EXIST.getMessage()); 
    if (resFileInfoTablePO == null || DCConst.TRUE.equals(resFileInfoTablePO.getIsDelete()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FILE_INFO_TABLE_NO_EXISTS.getCode()), StatusCodeEnum.DATA_RESOURCE_FILE_INFO_TABLE_NO_EXISTS.getMessage()); 
    Long dataSourceId = (updateFileInfoTableDTO.getDataSourceId() != null) ? updateFileInfoTableDTO.getDataSourceId() : resFileInfoTablePO.getDataSourceId();
    if (dataSourceId == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_SOURCE_IS_NULL.getCode()), StatusCodeEnum.DATA_RESOURCE_DATA_SOURCE_IS_NULL.getMessage()); 
    boolean fileTableNameChange = (!StringUtils.isEmpty(updateFileInfoTableDTO.getFileTableName()) && !updateFileInfoTableDTO.getFileTableName().equals(resFileInfoTablePO.getFileTableName()));
    boolean dataSourceIdChange = (updateFileInfoTableDTO.getDataSourceId() != null && !updateFileInfoTableDTO.getDataSourceId().equals(resFileInfoTablePO.getDataSourceId()));
    if (fileTableNameChange || dataSourceIdChange) {
      ResultEntity<Boolean> resultEntity = fileTableIsSync(resFileInfoTablePO, tokenId);
      if (resultEntity.getCode().intValue() != 0)
        return resultEntity; 
      List<ResFileInfoTablePO> list = ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).getTableByNameAndDataSourceId(resFileInfoTablePO.getDataSourceId(), updateFileInfoTableDTO.getFileTableName(), updateFileInfoTableDTO.getFileTableId());
      if (list.size() > 0)
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_NEW_FILE_TABLE_EN_NAME_EXISTS.getCode()), StatusCodeEnum.DATA_RESOURCE_NEW_FILE_TABLE_EN_NAME_EXISTS.getMessage()); 
      DataSourceVO dataSourceVO = null;
      if (dataSourceId != null)
        dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(dataSourceId, getDefaultCode(), tokenId).getObj(); 
      if (dataSourceVO == null)
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_FAILED.getMessage()); 
      if (DBKeywordUtil.isDBKeyword(updateFileInfoTableDTO.getFileTableName(), dataSourceVO.getType()))
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_EN_NAME_IS_KEYWORD.getCode()), StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_EN_NAME_IS_KEYWORD
            .getMessage()); 
      DataSourceDTO dataSourceDTO = new DataSourceDTO();
      BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
      ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
      if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
        return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
      DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
      if (checkHaveFileTableNameInDB(dataStructProvider, updateFileInfoTableDTO.getFileTableName()))
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_EXISTS_IN_DATA_SOURCE.getCode()), StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_EXISTS_IN_DATA_SOURCE.getMessage()); 
      if (fileTableNameChange)
        updateFileInfoTablePO.setFileTableName(updateFileInfoTableDTO.getFileTableName()); 
      if (dataSourceIdChange)
        updateFileInfoTablePO.setDataSourceId(updateFileInfoTableDTO.getDataSourceId()); 
    } 
    updateFileInfoTablePO.setFileTableId(updateFileInfoTableDTO.getFileTableId());
    updateFileInfoTablePO.setFileTableCode(updateFileInfoTableDTO.getFileTableCode());
    updateFileInfoTablePO.setFileTableOrder(updateFileInfoTableDTO.getFileTableOrder());
    updateFileInfoTablePO.setFileTableCnName(updateFileInfoTableDTO.getFileTableCnName());
    updateFileInfoTablePO.setDescription(updateFileInfoTableDTO.getDescription());
    updateFileInfoTablePO.setUpdateTime(new Date());
    if (((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).updateFileInfoTableSelective(updateFileInfoTablePO) > 0) {
      StringBuilder logInfo = new StringBuilder("修改文件信息表 ");
      if (!StringUtils.isEmpty(updateFileInfoTableDTO.getFileTableName()) && !updateFileInfoTableDTO.getFileTableName().equals(resFileInfoTablePO.getFileTableName()))
        logInfo.append("文件信息表英文名:【").append(resFileInfoTablePO.getFileTableName()).append("】修改为").append("【").append(updateFileInfoTableDTO.getFileTableName()).append("】;"); 
      if (!StringUtils.isEmpty(updateFileInfoTableDTO.getFileTableCnName()) && !updateFileInfoTableDTO.getFileTableCnName().equals(resFileInfoTablePO.getFileTableCnName()))
        logInfo.append("文件信息表中文名:【").append(resFileInfoTablePO.getFileTableCnName()).append("】修改为").append("【").append(updateFileInfoTableDTO.getFileTableCnName()).append("】;"); 
      logSystemInfo(logInfo.substring(0, logInfo.length() - 1), getDefaultCode(), tokenId);
      return ResultEntity.successMessage(Boolean.valueOf(true));
    } 
    return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_UPDATE_FILE_TABLE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_UPDATE_FILE_TABLE_FAILED.getMessage());
  }
  
  private boolean checkFileTableCodeExist(ResFileInfoTablePO fileInfoTablePO, UpdateFileInfoTableDTO updateFileInfoTableDTO) {
    if (updateFileInfoTableDTO.getFileTableCode().equals(fileInfoTablePO.getFileTableCode()))
      return false; 
    List<ResFileInfoTablePO> fileTableList = ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).getFileTableByCode(updateFileInfoTableDTO.getFileTableCode(), fileInfoTablePO.getFileTableId());
    if (fileTableList.size() > 0)
      return true; 
    return false;
  }
  
  private ResultEntity<Boolean> fileTableIsSync(ResFileInfoTablePO fileInfoTablePO, String tokenId) {
    if (fileInfoTablePO.getFileTableName() == null || fileInfoTablePO.getDataSourceId() == null)
      return ResultEntity.successMessage(Boolean.valueOf(false)); 
    try {
      DataSourceVO dataSourceVO = null;
      if (fileInfoTablePO.getDataSourceId() != null)
        dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(fileInfoTablePO.getDataSourceId(), getDefaultCode(), tokenId).getObj(); 
      if (dataSourceVO == null)
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_FAILED.getMessage()); 
      DataSourceDTO dataSourceDTO = new DataSourceDTO();
      BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
      ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
      if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
        return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
      DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
      Boolean existsTable = dataStructProvider.isExistsTable(fileInfoTablePO.getFileTableName());
      if (existsTable.booleanValue())
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_TABLE_HAS_SYNC_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_TABLE_HAS_SYNC_FAILED.getMessage()); 
      return ResultEntity.successMessage(Boolean.valueOf(false));
    } catch (Exception e) {
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CHECK_TABLE_SYNC_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_CHECK_TABLE_SYNC_FAILED.getMessage());
    } 
  }
  
  public ResultEntity<Object> delFileInfoTable(DeleteFileTableDTO deleteFileTableDTO, String tokenId) {
    if (!"fileTableId".equalsIgnoreCase(deleteFileTableDTO.getDeleteField()) && CollectionUtils.isEmpty(deleteFileTableDTO.getSelectKeys()))
      return ResultEntity.errorMessage(Integer.valueOf(-1), "删除文件表参数不正确"); 
    return delFileInfoTable(deleteFileTableDTO.getDirectoryId(), deleteFileTableDTO.getSelectKeys().get(0), tokenId);
  }
  
  public ResultEntity<Object> delFileInfoTable(Long directoryId, Long fileTableId, String tokenId) {
    List<String> deletedTableCnNames = new ArrayList<>();
    Map<Long, DataStructProvider> providerMap = new HashMap<>();
    boolean success = delFileInfoTable(fileTableId, providerMap, deletedTableCnNames, tokenId);
    if (success) {
      StringBuilder logInfo = new StringBuilder();
      logInfo.append("删除文件信息表 文件信息表【").append(deletedTableCnNames.get(0)).append("】");
      logSystemInfo(logInfo.toString(), getDefaultCode(), tokenId);
      DeleteFileTableMsg msg = new DeleteFileTableMsg();
      msg.setDirectoryId(directoryId);
      msg.setFileTableId(fileTableId);
      msg.setTokenId(tokenId);
      msg.setPlatCode(getDefaultCode());
      RabbitMQUtil.publish("DC.Resource.deleteFileTable", msg);
      return ResultEntity.successMessage(Boolean.valueOf(true));
    } 
    return ResultEntity.errorMessage(Integer.valueOf(-1), "删除文件表失败");
  }
  
  public ResultEntity<Object> delFileInfoTableByEntityId(Long entityId, Long fileTableId, String tokenId) {
    List<String> deletedTableCnNames = new ArrayList<>();
    Map<Long, DataStructProvider> providerMap = new HashMap<>();
    boolean success = delFileInfoTable(fileTableId, providerMap, deletedTableCnNames, tokenId);
    if (success) {
      StringBuilder logInfo = new StringBuilder();
      logInfo.append("删除文件信息表 文件信息表【").append(deletedTableCnNames.get(0)).append("】");
      logSystemInfo(logInfo.toString(), getDefaultCode(), tokenId);
      DeleteFileTableMsg msg = new DeleteFileTableMsg();
      msg.setEntityId(entityId);
      msg.setFileTableId(fileTableId);
      msg.setTokenId(tokenId);
      msg.setPlatCode(getDefaultCode());
      RabbitMQUtil.publish("DC.Resource.deleteFileTable", msg);
      return ResultEntity.successMessage(Boolean.valueOf(true));
    } 
    return ResultEntity.errorMessage(Integer.valueOf(-1), "删除文件表失败");
  }
  
  private boolean delFileInfoTable(Long fileTableId, Map<Long, DataStructProvider> providerMap, List<String> deletedTableCnNames, String tokenId) {
    ResFileInfoTablePO fileInfoTablePO = ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).selectFileInfoTable(fileTableId);
    if (fileInfoTablePO == null || DCConst.TRUE.equals(fileInfoTablePO.getIsDelete()))
      return true; 
    if (fileInfoTablePO.getDataSourceId() != null) {
      DataStructProvider dataStructProvider;
      if (providerMap.containsKey(fileInfoTablePO.getDataSourceId())) {
        dataStructProvider = providerMap.get(fileInfoTablePO.getDataSourceId());
      } else {
        DataSourceVO dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(fileInfoTablePO.getDataSourceId(), getDefaultCode(), tokenId).getObj();
        if (dataSourceVO == null) {
          logErrorInfo(null, "文件信息表【" + fileInfoTablePO.getFileTableName() + "】删除失败，根据Id获取配置的数据库实例失败，数据源id:【" + fileInfoTablePO.getDataSourceId() + "】", getDefaultCode(), tokenId);
          return false;
        } 
        DataSourceDTO dataSourceDTO = new DataSourceDTO();
        BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
        ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
        if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
          return false; 
        dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
        providerMap.put(fileInfoTablePO.getDataSourceId(), dataStructProvider);
      } 
      boolean isExistsTable = dataStructProvider.isExistsTable(fileInfoTablePO.getFileTableName()).booleanValue();
      boolean haveTableData = (isExistsTable && dataStructProvider.getTableDataCount(fileInfoTablePO.getFileTableName()) > 0);
      if (isExistsTable && !haveTableData) {
        boolean isSuccess = delFileInfoDbTable(dataStructProvider, fileInfoTablePO.getFileTableName());
        if (!isSuccess) {
          logErrorInfo(null, "文件信息表【" + fileInfoTablePO.getFileTableName() + "】删除失败，删除物理表失败", getDefaultCode(), tokenId);
          return false;
        } 
        try {
          String historyTableName = getHistoryTableName(fileInfoTablePO.getFileTableName());
          delFileInfoDbTable(dataStructProvider, historyTableName);
        } catch (Exception e) {
          logErrorInfo(ExceptionUtil.getStackTraceMsg(e), "文件信息表【" + fileInfoTablePO.getFileTableName() + "】删除物理表成功, 但是删除历史表失败", getDefaultCode(), tokenId);
        } 
      } 
    } 
    List<MetadataDirectoryPO> applyEntityListOfFileTable = getApplyEntityListOfFileTable(fileInfoTablePO.getFileTableId());
    if (applyEntityListOfFileTable.size() > 0) {
      IMetadataDao metadataDao = (IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class);
      for (MetadataDirectoryPO item : applyEntityListOfFileTable)
        metadataDao.delFileTableId(item.getDirectoryId()); 
    } 
    ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).deleteFileInfoTable(fileTableId);
    deletedTableCnNames.add(fileInfoTablePO.getFileTableCnName());
    return true;
  }
  
  private String getHistoryTableName(String fileTableName) {
    return fileTableName + "_history";
  }
  
  public ResultEntity<FileInfoTableVO> getFileTableByDirId(Long directoryId) {
    ResFileInfoTablePO fileInfoTable = ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).getFileTableByDirId(directoryId);
    if (fileInfoTable != null) {
      FileInfoTableVO fileInfoTableVO = new FileInfoTableVO();
      BeanUtils.copyProperties(fileInfoTable, fileInfoTableVO);
      return ResultEntity.successMessage(fileInfoTableVO);
    } 
    return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_NO_SEARCH_FILE_INFO.getCode()), StatusCodeEnum.DATA_RESOURCE_NO_SEARCH_FILE_INFO
        .getMessage());
  }
  
  public ResultEntity<FileInfoTableVO> getFileTableById(Long fileTableId) {
    ResFileInfoTablePO fileInfoTable = ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).getFileTableById(fileTableId);
    if (fileInfoTable != null) {
      FileInfoTableVO fileInfoTableVO = new FileInfoTableVO();
      BeanUtils.copyProperties(fileInfoTable, fileInfoTableVO);
      return ResultEntity.successMessage(fileInfoTableVO);
    } 
    return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_NO_SEARCH_FILE_INFO.getCode()), StatusCodeEnum.DATA_RESOURCE_NO_SEARCH_FILE_INFO
        .getMessage());
  }
  
  public ResultEntity<Boolean> saveFileTableId(Long directoryId, Long fileTableId, String tokenId) {
    IMetadataDao metadataDao = (IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class);
    IResourceFileInfoTableDao resFileInfoTableDao = (IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class);
    List<MetadataEntityPO> metadataEntitys = metadataDao.selectMetadataEntityByDirectoryId(directoryId);
    if (metadataEntitys.size() > 0) {
      MetadataDirectoryPO metadataDirectoryPO = metadataDao.selectDirectoryByDirectoryId(directoryId);
      if (metadataDirectoryPO == null)
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_DIRECTORY_NOT_EXISTS.getCode()), StatusCodeEnum.DATA_RESOURCE_DATA_DIRECTORY_NOT_EXISTS
            .getMessage()); 
      Long oldFileId = metadataDirectoryPO.getFileTableId();
      ResultEntity<List<InterfaceEntityRelationVO>> resultEntity = this.dataInterfaceClient.getInterfaceEntityRelation(((MetadataEntityPO)metadataEntitys.get(0)).getEntityId(), 
          getDefaultCode(), tokenId);
      if (resultEntity.getCode().intValue() != 0)
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_INTERFACE_REFERENCE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_INTERFACE_REFERENCE_FAILED
            .getMessage()); 
      if (((List)resultEntity.getObj()).size() > 0) {
        List<String> relationNameList = (List<String>)((List)resultEntity.getObj()).stream().map(InterfaceEntityRelationVO::getInterfaceName).collect(Collectors.toList());
        String message = "此数据表应用于接口【" + Joiner.on("、").join((Iterable)relationNameList) + "】，请先删除数据表在接口中的应用";
        return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_DATA_TABLE_APPLY_INTERFACE_FIRST_DELETE.getCode()), message);
      } 
      if (fileTableId != null) {
        metadataDirectoryPO.setFileTableId(fileTableId);
        metadataDirectoryPO.setUpdateTime(new Date());
        metadataDao.updateMetadataDirectory(metadataDirectoryPO);
      } else {
        metadataDao.delFileTableId(directoryId);
      } 
      StringBuilder logInfo = new StringBuilder();
      if (null != fileTableId) {
        ResFileInfoTablePO fileTable = resFileInfoTableDao.getFileTableById(fileTableId);
        if (fileTable != null) {
          String fileName = (fileTable.getFileTableCnName() != null) ? fileTable.getFileTableCnName() : fileTable.getFileTableName();
          if ("Manage".equals(getDefaultCode())) {
            logInfo.append("选择文件信息表 数据资源【");
          } else {
            logInfo.append("选择文件信息表 数据表【");
          } 
          logInfo.append(((MetadataEntityPO)metadataEntitys.get(0)).getEntityName()).append("】")
            .append("选择文件信息表【").append(fileName).append("】");
        } 
      } else {
        ResFileInfoTablePO fileTable = resFileInfoTableDao.getFileTableById(oldFileId);
        if (fileTable != null) {
          String fileName = (fileTable.getFileTableCnName() != null) ? fileTable.getFileTableCnName() : fileTable.getFileTableName();
          if ("Manage".equals(getDefaultCode())) {
            logInfo.append("取消文件信息表 数据资源【");
          } else {
            logInfo.append("取消文件信息表 数据表【");
          } 
          logInfo.append(((MetadataEntityPO)metadataEntitys.get(0)).getEntityName()).append("】")
            .append("取消文件信息表【").append(fileName).append("】");
        } 
      } 
      if (logInfo.length() > 0)
        logSystemInfo(logInfo.toString(), getDefaultCode(), tokenId); 
      return ResultEntity.successMessage(Boolean.valueOf(true));
    } 
    return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_NO_FOUND_DATA_RESOURCE_ENTITY.getCode()), StatusCodeEnum.DATA_RESOURCE_NO_FOUND_DATA_RESOURCE_ENTITY
        .getMessage());
  }
  
  private boolean delFileInfoDbTable(DataStructProvider dataStructProvider, String fileTableName) {
    return dataStructProvider.dropTable(fileTableName);
  }
  
  public boolean renameFileInfoDbTable(DataStructProvider dataStructProvider, String oldFileTableName, String newFileTableName) {
    return dataStructProvider.renameTableName(oldFileTableName, newFileTableName);
  }
  
  public ResultEntity<Boolean> createFileInfoDbTable(DataStructProvider dataStructProvider, String fileTableName, String tokenId) {
    ResultEntity<List<MetadataTypeVO>> result = this.dataStandardClient.getMetadataTypeList(getDefaultCode(), tokenId);
    if (result.getCode().intValue() != 0 || CollectionUtils.isEmpty((Collection)result.getObj())) {
      logErrorInfo(null, "创建 数据源 物理表失败! 获取全部数据元列表失败! 返回信息: " + JSON.toJSONString(result), getDefaultCode(), tokenId);
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CREATE_DATA_SOURCE_PHYSICAL_TABLE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_CREATE_DATA_SOURCE_PHYSICAL_TABLE_FAILED
          .getMessage());
    } 
    Map<String, MetadataTypeVO> metaTypeMap = new HashMap<>();
    ((List)result.getObj()).forEach(metadataTypeVO -> metaTypeMap.put(metadataTypeVO.getDataTypeName(), metadataTypeVO));
    Object object = new Object(this, metaTypeMap);
    boolean isGuid = DCConst.PrimaryKeyMode.GUID.equals(this.primaryKeyMode);
    if (isGuid) {
      object.add(new ColumnParamDTO(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.DBID), metaTypeMap.get("String"), COLUMN_LENGTH_36, null, 
            Integer.valueOf(1), Integer.valueOf(0)));
    } else {
      object.add(new ColumnParamDTO(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.DBID), metaTypeMap.get("Long"), 
            Integer.valueOf(0), null, Integer.valueOf(1), Integer.valueOf(0)));
    } 
    TableParamDTO table = new TableParamDTO();
    table.setTableName(fileTableName);
    table.setColumns((List)object);
    table.setSchemaName(dataStructProvider.currentDataSource.getDbSchema());
    return dataStructProvider.createTable(table, true);
  }
  
  public ResultEntity<Map<String, Integer>> handleOldDateForHistoryTable(String platCode, String tokenId) {
    Map<String, Integer> result = new HashMap<>();
    IResourceFileInfoTableDao resFileInfoTableDao = (IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class);
    Map<String, Object> param = new HashMap<>();
    List<ResFileInfoTablePO> fileInfoTableVOList = resFileInfoTableDao.selectFileInfoTableList(param);
    Integer successCount = Integer.valueOf(0);
    Integer failCount = Integer.valueOf(0);
    if (fileInfoTableVOList != null && !fileInfoTableVOList.isEmpty())
      for (ResFileInfoTablePO fileInfoTablePO : fileInfoTableVOList) {
        try {
          if (fileInfoTablePO == null || DCConst.TRUE.equals(fileInfoTablePO.getIsDelete()))
            continue; 
          DataSourceVO dataSourceVO = null;
          if (fileInfoTablePO.getDataSourceId() != null)
            dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(fileInfoTablePO.getDataSourceId(), platCode, tokenId).getObj(); 
          if (dataSourceVO == null || DCConst.TRUE.equals(dataSourceVO.getIsDelete()))
            continue; 
          DataStructProvider dataStructProvider = null;
          DataSourceDTO dataSourceDTO = new DataSourceDTO();
          BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
          ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
          if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
            continue; 
          dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
          this.historyTableManager.saveHistoryTableForFileTable(dataStructProvider, fileInfoTablePO.getFileTableId(), fileInfoTablePO.getFileTableName(), getDefaultCode(), tokenId, DCConst.DataResourceCategory.RES);
          Integer integer1 = successCount, integer2 = successCount = Integer.valueOf(successCount.intValue() + 1);
        } catch (Exception e) {
          logErrorInfo(ExceptionUtil.getStackTraceMsg(e), "同步文件信息表拉链表失败", getDefaultCode(), tokenId);
          Integer integer1 = failCount, integer2 = failCount = Integer.valueOf(failCount.intValue() + 1);
        } 
      }  
    result.put("successCount", successCount);
    result.put("failCount", failCount);
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<Boolean> checkDataSourceIdApplyFileInfoTable(Long dataSourceId) {
    return ResultEntity.successMessage(Boolean.valueOf((((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).selectFileInfoTableListCountByDataSourceId(dataSourceId) > 0)));
  }
  
  public ResultEntity<FileTableUsageVO> getFileTableUsedInfo(FileTableUsedInfoDTO usedInfoDTO, String platCode, String tokenId) {
    FileTableUsageVO result = new FileTableUsageVO();
    result.setDirectoryId(usedInfoDTO.getDirectoryId());
    result.setUsed(Boolean.valueOf(false));
    Long entityId = null;
    if ("Apply".equals(platCode)) {
      entityId = usedInfoDTO.getDirectoryId();
    } else {
      List<MetadataEntityPO> metadataEntityList = ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).selectMetadataEntityByDirectoryId(usedInfoDTO.getDirectoryId());
      if (null != metadataEntityList && metadataEntityList.size() > 0)
        entityId = ((MetadataEntityPO)metadataEntityList.get(0)).getEntityId(); 
    } 
    if (null == entityId)
      return ResultEntity.successMessage(result); 
    List<FileTableUsedInterfaceVO> fileTableUsedInterfaces = (List<FileTableUsedInterfaceVO>)this.dataInterfaceClient.fileTableUsedInterface(entityId, usedInfoDTO.getFileTableId(), DCConst.FALSE, "Collection", tokenId).getObj();
    if (CollectionUtils.isEmpty(fileTableUsedInterfaces))
      return ResultEntity.successMessage(result); 
    result.setUsed(Boolean.valueOf(true));
    result.setCollectionInterface(fileTableUsedInterfaces);
    return ResultEntity.successMessage(result);
  }
  
  public ResultEntity<Boolean> deleteFileTableUsage(FileTableUsedInfoDTO usedInfoDTO, String platCode, String tokenId) {
    Long entityId = null;
    if ("Apply".equals(platCode)) {
      entityId = usedInfoDTO.getDirectoryId();
    } else {
      List<MetadataEntityPO> metadataEntityList = ((IMetadataDao)ServiceUtil.getMetadataDao(platCode, IMetadataDao.class)).selectMetadataEntityByDirectoryId(usedInfoDTO.getDirectoryId());
      if (null != metadataEntityList && metadataEntityList.size() > 0)
        entityId = ((MetadataEntityPO)metadataEntityList.get(0)).getEntityId(); 
    } 
    this.dataInterfaceClient.removeInterfaceConfigByFileTableId(entityId, usedInfoDTO.getFileTableId(), "Collection", tokenId);
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  private List<MetadataDirectoryPO> getApplyEntityListOfFileTable(Long fileTableId) {
    return ((IMetadataDao)ServiceUtil.getMetadataDao(getDefaultCode(), IMetadataDao.class)).selectDirectoryListByFileTableId(fileTableId);
  }
  
  private ResultEntity<DataSourceDTO> getDataSourcePO(String platCode, Long dataSourceId, String tokenId) {
    DataSourceVO dataSourceVO = null;
    if (dataSourceId != null)
      dataSourceVO = (DataSourceVO)this.dataStandardClient.getDataSourceInfoDoNotDecryptById(dataSourceId, platCode, tokenId).getObj(); 
    if (dataSourceVO == null)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_BY_SYNC_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_GET_CONFIG_DB_INSTANCE_NAME_BY_SYNC_FAILED.getMessage()); 
    DataSourceDTO dataSourceDTO = new DataSourceDTO();
    BeanUtils.copyProperties(dataSourceVO, dataSourceDTO);
    return ResultEntity.successMessage(dataSourceDTO);
  }
  
  private void logErrorInfo(String stackInfo, String msg, String platCode, String tokenId) {
    CommonLogDTO logDTO = new CommonLogDTO();
    logDTO.setPlatCode(platCode);
    logDTO.setTokenId(tokenId);
    logDTO.setStackInformation(stackInfo);
    logDTO.setLoggerInformation(msg);
    logDTO.setLogLevel(FrameConst.Log.LOG_LEVEL_ERROR);
    logDTO.setLogType(0);
    logDTO.setModule_code(HeaderInterceptor.getModuleCode());
    RabbitMQUtil.publish("DC.Standard.log", logDTO);
  }
  
  private void logSystemInfo(String msg, String platCode, String tokenId) {
    CommonLogDTO logDTO = new CommonLogDTO();
    logDTO.setPlatCode(platCode);
    logDTO.setTokenId(tokenId);
    logDTO.setLoggerInformation(msg);
    logDTO.setLogLevel(FrameConst.Log.LOG_LEVEL_SYSTEM);
    logDTO.setLogType(0);
    logDTO.setModule_code(HeaderInterceptor.getModuleCode());
    RabbitMQUtil.publish("DC.Standard.log", logDTO);
  }
  
  public ResultEntity<Boolean> createResTableToDb(Long fileTableId, String tokenId) {
    ResFileInfoTablePO fileInfoTablePO = ((IResourceFileInfoTableDao)ServiceUtil.getResFileInfoTableDao(getDefaultCode(), IResourceFileInfoTableDao.class)).selectFileInfoTable(fileTableId);
    if (StringUtil.isEmpty(fileInfoTablePO.getFileTableName()))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_CHILD_ITEM_TO_DB_NO_TABLE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_CHILD_ITEM_TO_DB_NO_TABLE_FAILED.getMessage(), Boolean.valueOf(false)); 
    IResourceFileInfoFieldDao fileInfoFieldDao = (IResourceFileInfoFieldDao)ServiceUtil.getResFileInfoFieldDao(getDefaultCode(), IResourceFileInfoFieldDao.class);
    List<ResFileInfoFieldPO> fileInfoFieldList = fileInfoFieldDao.selectFileInfoFieldsByFileTableId(fileTableId);
    if (CollectionUtils.isEmpty(fileInfoFieldList))
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_FILE_FIELD_NULL_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_FILE_FIELD_NULL_FAILED.getMessage(), Boolean.valueOf(false)); 
    addMissingSysFields(fileInfoFieldList, fileTableId, tokenId);
    ResultEntity<DataSourceDTO> dataSourceResult = getDataSourcePO(getDefaultCode(), fileInfoTablePO.getDataSourceId(), tokenId);
    if (dataSourceResult.getCode().intValue() != 0)
      return (ResultEntity)dataSourceResult; 
    DataSourceDTO dataSourceDTO = (DataSourceDTO)dataSourceResult.getObj();
    ResultEntity<DataStructProvider> getDataBaseProviderRet = DataStructProvider.getDataBaseProvider(dataSourceDTO);
    if (!getDataBaseProviderRet.getCode().equals(Integer.valueOf(0)))
      return ResultEntity.errorMessage(getDataBaseProviderRet.getCode(), getDataBaseProviderRet.getMsg()); 
    DataStructProvider dataStructProvider = (DataStructProvider)getDataBaseProviderRet.getObj();
    try {
      String dataBaseType = dataSourceDTO.getType();
      Boolean tableIsExists = dataStructProvider.isExistsTable(fileInfoTablePO.getFileTableName());
      StringBuilder sbMsg = new StringBuilder();
      TableStructInfoDTO tableStructInfoDTO = getTableStructInfo(dataStructProvider, fileInfoTablePO, dataSourceDTO);
      if (!tableIsExists.booleanValue()) {
        ResultEntity<Boolean> createTableCheck = createTable(dataStructProvider, fileInfoTablePO, tokenId, fileInfoFieldList, dataBaseType, Boolean.valueOf(false));
        if (!createTableCheck.getCode().equals(Integer.valueOf(0)))
          return createTableCheck; 
      } else {
        Map<String, Object> updateInfoItemToDBRet = updateFileFieldToDB(fileInfoFieldList, tableStructInfoDTO, dataStructProvider, fileInfoTablePO, dataSourceDTO, getDefaultCode(), tokenId);
        sbMsg.append((StringBuilder)updateInfoItemToDBRet.get("sbMsg"));
      } 
      if (sbMsg.length() > 0)
        throw new Exception(sbMsg.toString()); 
      for (ResFileInfoFieldPO item : fileInfoFieldList) {
        ResFileInfoFieldPO updateFileInfoFieldPO = new ResFileInfoFieldPO();
        updateFileInfoFieldPO.setFieldId(item.getFieldId());
        updateFileInfoFieldPO.setFieldStatus(DCConst.TRUE);
        fileInfoFieldDao.updateFileInfoFieldSelective(updateFileInfoFieldPO);
        item.setFieldStatus(DCConst.TRUE);
      } 
      this.historyTableManager.saveHistoryTableForFileTable(dataStructProvider, fileInfoTablePO.getFileTableId(), fileInfoTablePO.getFileTableName(), getDefaultCode(), tokenId, DCConst.DataResourceCategory.RES);
    } catch (Exception ex) {
      logErrorInfo(ExceptionUtil.getStackTraceMsg(ex), "同步文件表字段到数据库方法异常", getDefaultCode(), tokenId);
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_FILE_FIELD_TO_DB_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_FILE_FIELD_TO_DB_FAILED.getMessage(), Boolean.valueOf(false));
    } 
    return ResultEntity.successMessage(Boolean.valueOf(true));
  }
  
  protected void addMissingSysFields(List<ResFileInfoFieldPO> fileInfoFieldList, Long fileTableId, String tokenId) {
    try {
      Set<String> fieldNames = (Set<String>)fileInfoFieldList.stream().map(ResFileInfoFieldPO::getFieldName).collect(Collectors.toSet());
      int maxOrder = ((Integer)fileInfoFieldList.stream().map(ResFileInfoFieldPO::getFieldOrder).filter(Objects::nonNull).max(Integer::compareTo).orElse(Integer.valueOf(1))).intValue();
      UserInfoDTO userInfo = TokenUtil.getUserInfoByTokenId(tokenId);
      IResourceFileInfoFieldDao fileInfoFieldDao = (IResourceFileInfoFieldDao)ServiceUtil.getResFileInfoFieldDao(getDefaultCode(), IResourceFileInfoFieldDao.class);
      if (!fieldNames.contains(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.SERVER_MODE))) {
        ResFileInfoFieldPO serverMode = new ResFileInfoFieldPO((Long)WorkerId.getNextId(Long.class), fileTableId, SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.SERVER_MODE), "文件服务器存储模式", "Int", Integer.valueOf(1), Integer.valueOf(0), Integer.valueOf(1), Integer.valueOf(0), Integer.valueOf(++maxOrder), userInfo.getUserId(), userInfo.getOrganizationId());
        fileInfoFieldDao.insertFileInfoField(serverMode);
        fileInfoFieldList.add(serverMode);
      } 
    } catch (Exception e) {
      LogHelper.error("补充缺失内置字段异常", ExceptionUtil.getStackTraceMsg(e));
    } 
  }
  
  private TableStructInfoDTO getTableStructInfo(DataStructProvider dataStructProvider, ResFileInfoTablePO fileInfoTable, DataSourceDTO dataSourceDTO) {
    TableStructInfoDTO tableStructInfoDTO = new TableStructInfoDTO();
    Map<String, String> tableStruct = new HashMap<>();
    Map<String, Integer> tableIsNull = new HashMap<>();
    List<String> fieldList = new ArrayList<>();
    List<String> pkFields = new ArrayList<>();
    List<DBColumnDTO> tableStructDt = dataStructProvider.getEntityDbTableStruct(fileInfoTable.getFileTableName(), null);
    if (tableStructDt != null && tableStructDt.size() > 0)
      tableStructDt.stream().forEach(item -> {
            tableStruct.put(item.getColumnName().toLowerCase(), item.getDataType());
            tableIsNull.put(item.getColumnName().toLowerCase(), item.getIsNullAble());
            fieldList.add(item.getColumnName());
            if (Objects.equals(item.getIsPrimaryKey(), DCConst.TRUE))
              pkFields.add(item.getColumnName()); 
          }); 
    tableStructInfoDTO.setFieldList(fieldList);
    tableStructInfoDTO.setPkFields(pkFields);
    tableStructInfoDTO.setTableIsNull(tableIsNull);
    tableStructInfoDTO.setTableStruct(tableStruct);
    return tableStructInfoDTO;
  }
  
  private ResultEntity createTable(DataStructProvider dataStructProvider, ResFileInfoTablePO fileInfoTable, String tokenId, List<ResFileInfoFieldPO> fileInfoFieldList, String dataBaseType, Boolean tableIsExists) {
    ResultEntity<Boolean> checkResult = this.historyTableManager.checkHistoryTableName(dataStructProvider, fileInfoTable.getFileTableName(), getDefaultCode(), DCConst.TableType.FILETABLE);
    if (checkResult.getCode().intValue() != 0)
      return checkResult; 
    ResultEntity<Boolean> createTableForSysToDbRet = createTableForSysToDb(fileInfoFieldList, getDefaultCode(), dataBaseType, dataStructProvider, fileInfoTable, tokenId);
    if (!createTableForSysToDbRet.getCode().equals(Integer.valueOf(0)))
      return createTableForSysToDbRet; 
    tableIsExists = (Boolean)createTableForSysToDbRet.getObj();
    if (!tableIsExists.booleanValue())
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_CREATE_DB_TABLE_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_CREATE_DB_TABLE_FAILED.getMessage(), Boolean.valueOf(false)); 
    return ResultEntity.successMessage(null);
  }
  
  private ResultEntity<Boolean> createTableForSysToDb(List<ResFileInfoFieldPO> fileInfoFieldCol, String platCode, String dataBaseType, DataStructProvider dataStructProvider, ResFileInfoTablePO entity, String tokenId) {
    int noPKCount = 0;
    int longTypeColumnCount = 0;
    TableParamDTO table = new TableParamDTO();
    List<ColumnParamDTO> columns = new ArrayList<>();
    for (int i = 0; i < fileInfoFieldCol.size(); i++) {
      ResFileInfoFieldPO fieldItem = fileInfoFieldCol.get(i);
      columns.add(createColumnParamDTO(fieldItem, dataBaseType, platCode, tokenId, SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID).equalsIgnoreCase(fieldItem.getFieldName()) ? DCConst.TRUE : DCConst.FALSE, fieldItem.getIsRequired()));
      if (Objects.equals(DCConst.TRUE, fieldItem.getIsPrimaryKey())) {
        if ("sqlserver".equalsIgnoreCase(dataBaseType)) {
          if (fieldItem.getFieldLength() != null && fieldItem.getFieldLength().intValue() < 0)
            return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FIELD_TYPE_CANNOT_SET_PRIMARY.getCode()), String.format(StatusCodeEnum.DATA_RESOURCE_FIELD_TYPE_CANNOT_SET_PRIMARY.getMessage(), new Object[] { fieldItem.getFieldName(), fieldItem.getFieldType() + "(max)" })); 
        } else if ("oracle".equalsIgnoreCase(dataBaseType)) {
          if (fieldItem.getFieldLength().intValue() >= 2000)
            return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_FIELD_TYPE_CANNOT_SET_PRIMARY.getCode()), String.format(StatusCodeEnum.DATA_RESOURCE_FIELD_TYPE_CANNOT_SET_PRIMARY.getMessage(), new Object[] { fieldItem.getFieldName(), fieldItem.getFieldType() + "(2000)" })); 
          if ("Long".equalsIgnoreCase(fieldItem.getFieldType()))
            return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_PK_FIELD_CANNOT_LONG_FAILED.getCode()), String.format(StatusCodeEnum.DATA_RESOURCE_FIELD_TYPE_CANNOT_SET_PRIMARY.getMessage(), new Object[] { fieldItem.getFieldName(), "long" })); 
        } 
        if (!SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equals(fieldItem.getFieldName()))
          noPKCount++; 
      } 
      if ("oracle".equalsIgnoreCase(dataBaseType))
        if ("Long".equalsIgnoreCase(fieldItem.getFieldType()))
          longTypeColumnCount++;  
    } 
    if (longTypeColumnCount > 1)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ONLY_ONE_LONG_TYPE_FIELD_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_ONLY_ONE_LONG_TYPE_FIELD_FAILED.getMessage()); 
    if (noPKCount == 0)
      return ResultEntity.errorMessage(Integer.valueOf(StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_NO_PK_FAILED.getCode()), StatusCodeEnum.DATA_RESOURCE_SYNC_ENTITY_NO_PK_FAILED.getMessage()); 
    table.setTableName(entity.getFileTableName());
    table.setColumns(columns);
    table.setSchemaName(dataStructProvider.currentDataSource.getDbSchema());
    return ResultEntity.successMessage(dataStructProvider.createTable(table));
  }
  
  public ColumnParamDTO createColumnParamDTO(ResFileInfoFieldPO fileInfoFieldPO, String dbType, String platCode, String tokenId, Integer isPrimaryKey, Integer isRequired) {
    ColumnParamDTO newColumn = new ColumnParamDTO();
    newColumn.setColumnName(fileInfoFieldPO.getFieldName());
    newColumn.setColumnScale(fileInfoFieldPO.getFieldScale());
    newColumn.setColumnLength(fileInfoFieldPO.getFieldLength());
    if (DCConst.FALSE.equals(isRequired)) {
      newColumn.setIsNull(DCConst.TRUE);
    } else {
      newColumn.setIsNull(DCConst.FALSE);
    } 
    MetadataTypeVO metadataTypeVO = (MetadataTypeVO)this.dataStandardClient.getMetadataTypeByDataTypeName(fileInfoFieldPO.getFieldType(), platCode, tokenId).getObj();
    if (SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.DBID).equalsIgnoreCase(fileInfoFieldPO.getFieldName()))
      if (dbType.equalsIgnoreCase("postgresql")) {
        metadataTypeVO.setPgsqlDbType(getPrimaryKeyType(dbType, metadataTypeVO.getPgsqlDbType()));
      } else if (dbType.equalsIgnoreCase("dm")) {
        metadataTypeVO.setDm8DbType(getPrimaryKeyType(dbType, metadataTypeVO.getDm8DbType()));
      } else if (dbType.equalsIgnoreCase("kingbase")) {
        metadataTypeVO.setKingBaseDbType(getPrimaryKeyType(dbType, metadataTypeVO.getKingBaseDbType()));
      } else if (dbType.equalsIgnoreCase("mysql")) {
        metadataTypeVO.setMysqlDbType(getPrimaryKeyType(dbType, metadataTypeVO.getMysqlDbType()));
      } else if (dbType.equalsIgnoreCase("sqlserver")) {
        metadataTypeVO.setSqlDbType(getPrimaryKeyType(dbType, metadataTypeVO.getSqlDbType()));
      } else if (dbType.equalsIgnoreCase("oracle")) {
        metadataTypeVO.setOracleDbType(getPrimaryKeyType(dbType, metadataTypeVO.getOracleDbType()));
      }  
    if (SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.START_TIME).equalsIgnoreCase(newColumn.getColumnName()) || 
      SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.LAST_MODIFY_TIME).equalsIgnoreCase(newColumn.getColumnName()) || 
      SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ACCESS_DATE).equalsIgnoreCase(newColumn.getColumnName()))
      if ("mysql".equals(dbType)) {
        metadataTypeVO.setMysqlDbType("datetime");
        newColumn.setColumnLength(Integer.valueOf(3));
      } else if ("sqlserver".equals(dbType)) {
        metadataTypeVO.setSqlDbType("datetime2");
        newColumn.setColumnLength(Integer.valueOf(3));
      }  
    newColumn.setColumnType(metadataTypeVO);
    if (Objects.equals(DCConst.TRUE, isPrimaryKey)) {
      if (SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID).equalsIgnoreCase(fileInfoFieldPO.getFieldName())) {
        newColumn.setIsPrimaryKey(DCConst.TRUE);
      } else {
        newColumn.setIsNull(DCConst.FALSE);
        newColumn.setIsPrimaryKey(DCConst.FALSE);
      } 
    } else {
      newColumn.setIsPrimaryKey(DCConst.FALSE);
    } 
    return newColumn;
  }
  
  private String getPrimaryKeyType(String dbType, String columnType) {
    boolean isGuid = DCConst.PrimaryKeyMode.GUID.equals(this.primaryKeyMode);
    if (dbType.equals("postgresql")) {
      if (isGuid) {
        columnType = "character varying(36)";
      } else {
        columnType = "bigint";
      } 
    } else if (dbType.equals("kingbase")) {
      if (isGuid) {
        columnType = "character(36)";
      } else {
        columnType = "bigint";
      } 
    } else if (dbType.equals("dm")) {
      if (isGuid) {
        columnType = "varchar(36)";
      } else {
        columnType = "bigint";
      } 
    } else if (dbType.equals("sqlserver")) {
      if (isGuid) {
        columnType = "nvarchar(36)";
      } else {
        columnType = "bigint";
      } 
    } else if (isGuid) {
      columnType = "char(36)";
    } else {
      columnType = "bigint";
    } 
    return columnType;
  }
  
  public Boolean setDBFieldDesc(TableStructInfoDTO tableStructInfoDTO, List<ResFileInfoFieldPO> fileInfoFieldPOList, DataStructProvider dataStructProvider, String tableName, String platCode, String tokenId) {
    Boolean result = Boolean.valueOf(true);
    try {
      Map<String, String> tableStruct = tableStructInfoDTO.getTableStruct();
      for (ResFileInfoFieldPO item : fileInfoFieldPOList) {
        String columnType = getColumnType(tableStruct, item, dataStructProvider.getDbType(), platCode, tokenId);
        if (item.getFieldName().equalsIgnoreCase(SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.DBID))) {
          boolean isGuid = DCConst.PrimaryKeyMode.GUID.equals(this.primaryKeyMode);
          if (isGuid)
            if (dataStructProvider.getDbType().equals("postgresql")) {
              columnType = "character varying(36)";
            } else if (dataStructProvider.getDbType().equals("kingbase")) {
              columnType = "character(36)";
            } else if (dataStructProvider.getDbType().equals("dm")) {
              columnType = "varchar(36)";
            } else if (dataStructProvider.getDbType().equals("sqlserver")) {
              columnType = "nvarchar(36)";
            } else {
              columnType = "char(36)";
            }  
        } 
        if (!dataStructProvider.setColumnDescription(item.getFieldName(), item.getFieldType(), item.getFieldLength(), item.getFieldScale(), item.getDescription(), dataStructProvider.currentDataSource.getDbSchema(), tableName, columnType, item.getIsPrimaryKey(), item.getIsRequired()))
          result = Boolean.valueOf(false); 
      } 
    } catch (Exception ex) {
      logErrorInfo(ExceptionUtil.getStackTraceMsg(ex), "设置字段描述信息方法异常", platCode, tokenId);
    } 
    return result;
  }
  
  private Map<String, Object> updateFileFieldToDB(List<ResFileInfoFieldPO> fileInfoFieldPOList, TableStructInfoDTO tableStructInfoDTO, DataStructProvider dataStructProvider, ResFileInfoTablePO fileInfoTable, DataSourceDTO dataSourceDTO, String platCode, String tokenId) {
    Map<String, Object> result = new HashMap<>();
    StringBuilder sbMsg = new StringBuilder();
    List<String> fieldList = tableStructInfoDTO.getFieldList();
    Map<String, String> tableStruct = tableStructInfoDTO.getTableStruct();
    List<String> errorFields = new ArrayList<>();
    for (ResFileInfoFieldPO item : fileInfoFieldPOList) {
      try {
        String dataItemName = item.getFieldName();
        if (!tableStructInfoDTO.getTableStruct().containsKey(dataItemName.toLowerCase())) {
          String columnType = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnType(dataSourceDTO.getType(), item.getFieldType(), platCode, tokenId);
          Boolean isAdd = dataStructProvider.createColumn(fileInfoTable.getFileTableName(), dataItemName, columnType, item.getFieldLength(), item.getFieldScale(), (item.getIsRequired().intValue() == 1));
          if (!isAdd.booleanValue())
            throw new Exception("创建列sql执行异常"); 
          fieldList.add(dataItemName);
        } else if (!SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID).equalsIgnoreCase(dataItemName) && !SysFieldHelper.getBaseSysFieldName(BaseSysFieldEnum.IS_DELETE).equalsIgnoreCase(dataItemName)) {
          String dataType = getColumnType(tableStruct, item, dataSourceDTO.getType(), platCode, tokenId);
          MetadataFieldVO metadataFieldVO = new MetadataFieldVO();
          metadataFieldVO.setDataItemName(item.getFieldName());
          metadataFieldVO.setIsPrimaryKey(item.getIsPrimaryKey());
          String schemaName = dataStructProvider.currentDataSource.getDbName();
          if (dataStructProvider.currentDataSource.getType().equalsIgnoreCase("sqlserver") || dataStructProvider.currentDataSource.getType().equalsIgnoreCase("dm"))
            schemaName = dataStructProvider.currentDataSource.getDbSchema(); 
          Boolean isAlter = Boolean.valueOf(dataStructProvider.alterColumn(schemaName, fileInfoTable.getFileTableName(), (item.getFieldLength() == null) ? 0 : item.getFieldLength().intValue(), (item.getFieldScale() == null) ? 0 : item.getFieldScale().intValue(), dataType, (item.getIsRequired().intValue() == 1) ? 0 : 1, metadataFieldVO));
          if (!isAlter.booleanValue())
            throw new Exception("修改列sql执行异常"); 
        } 
        if (SysFieldHelper.getFileSysFieldName(FileSysFieldEnum.ID).equalsIgnoreCase(item.getFieldName()))
          dataStructProvider.setTablePK(fileInfoTable.getFileTableName(), item.getFieldName()); 
        if (!setDBFieldDesc(tableStructInfoDTO, Arrays.asList(new ResFileInfoFieldPO[] { item }, ), dataStructProvider, fileInfoTable.getFileTableName(), platCode, tokenId).booleanValue())
          throw new Exception("修改列sql备注执行异常"); 
      } catch (Exception ex) {
        errorFields.add(item.getFieldName());
        String msg = "同步字段【" + item.getFieldName() + "】到数据库失败" + (StringUtils.hasText(ex.getMessage()) ? ("，原因：" + ex.getMessage()) : "同步到数据库失败");
        logErrorInfo(ExceptionUtil.getStackTraceMsg(ex), msg, platCode, tokenId);
      } 
    } 
    if (!errorFields.isEmpty())
      sbMsg.append("文件表字段【" + String.join(",", (Iterable)errorFields) + "】同步失败"); 
    result.put("fieldList", fieldList);
    result.put("sbMsg", sbMsg);
    return result;
  }
  
  private String getColumnType(Map<String, String> tableStruct, ResFileInfoFieldPO fileInfoField, String dbType, String platCode, String tokenId) {
    List<String> typeMapping = ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnTypeMapping(dbType, fileInfoField.getFieldType(), platCode, tokenId);
    if (typeMapping != null && typeMapping.contains(tableStruct.get(fileInfoField.getFieldName())))
      return tableStruct.get(fileInfoField.getFieldName()); 
    return ((IInfoItemSync)ServiceUtil.getInfoItemSyncManager(getDefaultCode(), IInfoItemSync.class)).getDBColumnType(dbType, fileInfoField.getFieldType(), platCode, tokenId);
  }
}
